require File.expand_path(File.dirname(__FILE__) + '/spec_helper')

describe "NormalizeIt" do
  describe "classes" do
    it 'should hook in to AR:Base' do
      ActiveRecord::Base.should respond_to :has_normalized
      ActiveRecord::Base.should respond_to :normalizes
    end

    it "should error if a static lookup class is defined without specifying which field is being normalized" do
      expect { class NewField < ActiveRecord::Base
                 normalizes :customers, :allow_inserts => false
               end
             }.should raise_error NormalizeIt::NormalizeItException
    end
  end

  describe "instances" do
    it "should not allow creating a new customer_status" do
      customer_status = CustomerStatus.new(:customer_status => 'old')
      customer_status.save
      customer_status.errors.should == {:base=>["Creation of CustomerStatus not allowed!"]}
      expect { customer_status.save! }.should raise_error ActiveRecord::RecordNotSaved
    end

    it "should create a new customer by passing in normalized values through attributes" do
      customer = Customer.new(:name => 'Dave Havlicek', :email_address => 'xens@comcast.net', :customer_status => 'new')
      customer.save
      Customer.should have(1).records
      customer.email_address.should == 'xens@comcast.net'
      customer.customer_status.should == 'new'
    end

    it "should create a new customer by calling new and then assigning attributes" do
      customer = Customer.new(:name => 'Dave Havlicek')
      customer.email_address = 'xens@comcast.net'
      customer.customer_status = 'new'
      customer.save
      Customer.should have(1).records
      customer.email_address.should == 'xens@comcast.net'
      customer.customer_status.should == 'new'
    end

    it "should use already-existing objects for attributes" do
      email_address = EmailAddress.create(:email_address => 'xens@comcast.net')
      customer_status = CustomerStatus.find_by_customer_status 'new'
      customer = Customer.new(:name => 'Dave Havlicek', :email_address => 'xens@comcast.net', :customer_status => 'new')
      customer.save
      Customer.should have(1).records
      customer.email_address_id.should == email_address.id
      customer.customer_status_id.should == customer_status.id
    end

    it "should not fail if normalized objects are created before the original object is saved" do
      customer = Customer.new(:name => 'Dave Havlicek', :email_address => 'xens@comcast.net', :customer_status => 'new')
      email_address = EmailAddress.create(:email_address => 'xens@comcast.net')
      customer_status = CustomerStatus.find_by_customer_status 'new'
      customer.save
      Customer.should have(1).records
      customer.email_address_id.should == email_address.id
      customer.customer_status_id.should == customer_status.id
    end

    it "should be able to use a rails-like finder to find objects by normalized attributes" do
      customer = Customer.new(:name => 'Dave Havlicek', :email_address => 'xens@comcast.net', :customer_status => 'new')
      customer.save
      customers = Customer.find_all_by_email_address 'xens@comcast.net'
      customers.size.should == 1
      customers.first.should == customer
    end

    it "should be able to combine rails-like finders" do
      pending "it would be nice to have this feature someday" do
        customer = Customer.new(:name => 'Dave Havlicek', :email_address => 'xens@comcast.net', :customer_status => 'new')
        customer.save
        customer_1 = Customer.find_by_email_address_and_customer_status_and_name 'xens@comcast.net', 'new', 'Dave Havlicek'
        customer_1.should == customer
      end
    end

    it "should not break original rails finders" do
      customer = Customer.new(:name => 'Dave Havlicek', :email_address => 'xens@comcast.net', :customer_status => 'new')
      customer.save
      customer_1 = Customer.find_by_name 'Dave Havlicek'
      customer_1.should == customer
    end

    it "should allow referencing static lookup models with [] notation" do
      customer_status = CustomerStatus.find_by_customer_status 'new'
      CustomerStatus[:new].should == customer_status
      CustomerStatus['new'].should == customer_status
    end
  end
end
